/* $Id: main.c,v 2.0 1998/03/25 05:43:54 svmcguir Exp svmcguir $ */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "path.h"
#include "option.h"
#include "mesh.h"
#include "util.h"
#include "inner.h"
#include "blackScholes.h"

#define PHRASE "smuggle munitions KGB NORAD Albanian Serbian Ortega cracking radar"

void weightMeasure(meshData *md, optionData *od, pathData *pd);

void option_error(char *argv[]);

void init(meshData *md, optionData *od, pathData *pd, int argc,
                   char *argv[]);


int main(int argc, char *argv[]){
  meshData md; 
  optionData od;
  pathData pd;
  int i,j;
  int time, path, underlier, reps;
  int cmac, cpac; 
  double  *cme, *cpe;
  double  cma, cmv, cpa, cpv;
  int toPath, fromPath;
  double weightSum, weightSqSum; 

  init(&md, &od, &pd, argc, argv);

  allocate1dDouble(&cme,md.reps);  /* controlled mesh estimate */
  allocate1dDouble(&cpe,md.reps);  /* controlled path estimate */

  /* Initialize means and variances for various estimates */
  cma  = 0;
  cpa = 0;
  cmv = 0;
  cpv = 0;
  for(reps=0;reps<md.reps;reps++){ 

    pathDataFill(&pd);

    optionDataFill(&od,&pd);

    meshSetFinalValue(&md, &od, &pd);

    meshDataFill(&md, &od, &pd, md.innerControl);

    /* store values of controlled mesh and path estimates */
    cme[reps] = md.valueIfHeld[0][0];
    cpe[reps] = innerControlledPathEstimate(&md,&od,&pd,md.innerControl);

    cma += cme[reps];
    cmv += cme[reps]*cme[reps];
    cpa += cpe[reps];
    cpv += cpe[reps]*cpe[reps];

    printf("rep=%d: cme=%lf\t cpe=%lf\n",reps,cme[reps],cpe[reps]);

    /* Output to weightData file */
    weightMeasure(&md, &od, &pd); 
    for(time=0;time<pd.times;time++){   
      fprintf(md.weightfile,"%lg\t%lg\t%lg\t%lg\n",(double)time,md.weightMeasure1[time],md.weightMeasure2[time],md.weightMeasure3[time]);   
    }   


    /* Output to errorData file */
/*     fprintf(md.errorfile,"time, path, "); */
/*     for(underlier=0;underlier<pd.underliers;underlier++){ */
/*       fprintf(md.errorfile,"p%i, ",underlier); */
/*     } */
/*     fprintf(md.errorfile,"err, err/val\n"); */
/*     fprintf(md.errorfile,"----------\n"); */

/*     for(time=0; time<pd.times; time++){ */
/*       for(path=0; path<md.meshPaths; path++){ */
/*         fprintf(md.errorfile,"%10i, %10i, ",time,path); */
/*         for(underlier=0; underlier<pd.underliers; underlier++){ */
/*           fprintf(md.errorfile,"%10lf, ",pd.price[time][path][underlier]); */
/*         } */
/*         fprintf(md.errorfile,"%10lf, %10lf\n",sqrt(md.variance[time][path]), */
/*                 sqrt(md.variance[time][path])/md.valueIfHeld[time][path]); */

        
/*       } */
/*     } */

  }

  printf("\n");
  cma /= md.reps;
  cmv = cmv/md.reps - cma*cma;
  cpa /= md.reps;
  cpv = cpv/md.reps - cpa*cpa;
  printf("%lf\t%lf\n",cma,cpa);
  printf("%lf\t%lf\n",sqrt(cmv),sqrt(cpv));
  printf("%lf\t%lf\n",sqrt(cmv/md.reps),sqrt(cpv/md.reps));

  fclose(md.weightfile);
  fclose(md.errorfile);
}

void weightMeasure(meshData *md, optionData *od, pathData *pd){
  int time, fromPath, toPath;
  double weightSum, weightSqSum, maxWeight, w;

  for(time=0; time <pd->times-1; time++){
    md->weightMeasure1[time] = 0;
    md->weightMeasure2[time] = 0;
    md->weightMeasure3[time] = 0;
    for(fromPath=0; fromPath<md->meshPaths; fromPath++){
      weightSum = 0;
      weightSqSum = 0;
      maxWeight = 0;
      for(toPath=0; toPath<md->meshPaths; toPath++){
        w = weight(md, od, pd, time, pd->price[time][fromPath], toPath);
        weightSum += w;
        weightSqSum += w*w;
        if(w > maxWeight) maxWeight = w;
      }
      md->weightMeasure1[time] += weightSum*weightSum/weightSqSum;
      md->weightMeasure2[time] += weightSum/maxWeight;
      md->weightMeasure3[time] += weightSum;

/*       printf("%d\t%d\t %lg\n",time,fromPath, */
/*               weightSum/maxWeight); */


    }
    md->weightMeasure1[time] /= md->meshPaths;
    md->weightMeasure2[time] /= md->meshPaths;
    md->weightMeasure3[time] /= md->meshPaths;

  }
}
      
void init(meshData *md, optionData *od, pathData *pd, int argc,
                   char *argv[]){
  int i,j,path,underlier;
  const char *options = "n:r:D:v:T:K:S:N:b:p:d:c:I:e:F:";
  int n,N,b,p,d,option,count=0,c,I;
  double r,D,v,K,S,T,e,F;
  
  /* Defaults */
  n = 5;
  r = 0.05;
  D = 0.1;
  v = 0.04;
  T = 3;
  K = 100;
  S = 90;
  N = 50;
  b = 50;
  p = 500;
  d = 3;
  c = 1;
  I = 3;
  e = 3;
  F = 0.0001;
  
  
  /* - - - - - Read command line parameters - - - - - */
  while((option = getopt(argc,argv,options)) != EOF){
    switch(option){
    case('e'):
      sscanf(optarg,"%lf",&e);
      count++;
      break;
    case('F'):
      sscanf(optarg,"%lf",&F);
      count++;
      break;
    case('I'):  
      sscanf(optarg,"%i",&I);
      count++;
      break;
    case('n'):  
      sscanf(optarg,"%i",&n);
      count++;
      break;
    case('r'):
      sscanf(optarg,"%lf",&r);
      count++;
      break;
    case('D'):
      sscanf(optarg,"%lf",&D);
      count++;
      break;
    case('v'):
      sscanf(optarg,"%lf",&v);
      count++;
      break;
    case('T'):
      sscanf(optarg,"%lf",&T);
      count++;
      break;
    case('K'):
      sscanf(optarg,"%lf",&K);
      count++;
      break;
    case('S'):
      sscanf(optarg,"%lf",&S);
      count++;
      break;
    case('N'):
      sscanf(optarg,"%i",&N);
      count++;
      break;
    case('b'):
      sscanf(optarg,"%i",&b);
      count++;
      break;
    case('p'):
      sscanf(optarg,"%i",&p);
      count++;
      break;
    case('d'):
      sscanf(optarg,"%i",&d);
      count++;
      break;
    case('c'):
      sscanf(optarg,"%i",&c);
      count++;
      break;
    default:
      option_error(argv);
    }
  }

  printf("Values input \n");
  printf("Number of underliers         n = %i\n",n);
  printf("Final time                   T = %lf\n",T);
  printf("Total number of repetitions  N = %i\n",N);
  printf("Number of mesh points        b = %i\n",b);
  printf("Number of paths              p = %i\n",p);
  printf("Number of time slices        d = %i\n",d);
  printf("Risk-free interest rate      r = %lf\n",r);
  printf("Dividend rate                D = %lf\n",D);
  printf("Variance                     v = %lf\n",v);
  printf("Strike price                 K = %lf\n",K);
  printf("Initial strike price         S = %lf\n",S);
  printf("Number of paths in a cluster c = %i\n",c);
  printf("Using control                I = %i\n",I);
  printf("Error limit factor           e = %lf\n",e);
  printf("Error limit floor            F = %lf\n",F);

  md->weightfile = fopen("weightData","a");
  md->errorfile = fopen("errorData","a");

  fprintf(md->weightfile,"#n = %i\n",n);
  fprintf(md->weightfile,"#T = %i\n",T);
  fprintf(md->weightfile,"#N = %i\n",N);
  fprintf(md->weightfile,"#b = %i\n",b);
  fprintf(md->weightfile,"#p = %i\n",p);
  fprintf(md->weightfile,"#d = %i\n",d);
  fprintf(md->weightfile,"#r = %lf\n",r);
  fprintf(md->weightfile,"#D = %lf\n",D);
  fprintf(md->weightfile,"#v = %lf\n",v);
  fprintf(md->weightfile,"#K = %lf\n",K);
  fprintf(md->weightfile,"#S = %lf\n",S);
  fprintf(md->weightfile,"#c = %i\n",c);
  fprintf(md->weightfile,"#I = %i\n",I);
  fprintf(md->weightfile,"#e = %lf\n",e);
  fprintf(md->weightfile,"#F = %lf\n",F);

  fprintf(md->errorfile,"#n = %i\n",n);
  fprintf(md->errorfile,"#T = %i\n",T);
  fprintf(md->errorfile,"#N = %i\n",N);
  fprintf(md->errorfile,"#b = %i\n",b);
  fprintf(md->errorfile,"#p = %i\n",p);
  fprintf(md->errorfile,"#d = %i\n",d);
  fprintf(md->errorfile,"#r = %lf\n",r);
  fprintf(md->errorfile,"#D = %lf\n",D);
  fprintf(md->errorfile,"#v = %lf\n",v);
  fprintf(md->errorfile,"#K = %lf\n",K);
  fprintf(md->errorfile,"#S = %lf\n",S);
  fprintf(md->errorfile,"#c = %i\n",c);
  fprintf(md->errorfile,"#I = %i\n",I);
  fprintf(md->errorfile,"#e = %lf\n",e);
  fprintf(md->errorfile,"#F = %lf\n",F);

  pd->times = d + 1;
  pd->paths = p;
  pd->underliers = n;
  pd->clusterSize = c;
  od->strikePrice = K;
  od->riskFreeRate = r*(double)T/(double)d;
  md->meshPaths = b;
  pd->meshPaths = b;
  md->reps = N;
  md->innerControl = I;
  strcpy(pd->phrase,PHRASE);

  pathDataAllocate(pd);
  optionDataAllocate(od,pd);
  meshDataAllocate(md,od,pd);

  for(i=0;i<pd->underliers;i++){
    for(j=0;j<pd->underliers;j++){
      if(i == j){
        pd->covariance[i][j] = v*(double)T/(double)d;
        pd->growthRate[i] = r*(double)T/(double)d;
        pd->dividendRate[i] = D*(double)T/(double)d;
      }else{
        pd->covariance[i][j] = 0;
      }
    }
  }

  pathParamsInit(pd);

  for(path=0; path<pd->meshPaths; path++){
    for(underlier=0; underlier < pd->underliers; underlier++){
      pd->price[0][path][underlier] = S;
    }
  }
}

void option_error(char *argv[]){
  printf("Usage: %s parameters\n",argv[0]);
  printf("\t-n underliers (n from Broadie and Glasserman)\n");
  printf("\t-r risk-free rate (r)\n");
  printf("\t-D dividend rate (\\delta)\n");
  printf("\t-v variance (\\sigmna^2)\n");
  printf("\t-T time to maturity (T)\n");
  printf("\t-K strikeprice (K)\n");
  printf("\t-S initial price (S_0)\n");
  printf("\t-N repetitions (N)\n");
  printf("\t-b paths in mesh (b)\n");
  printf("\t-p paths in path estimate (n_p)\n");
  printf("\t-d time slices (d)\n");
  printf("\t-c number of paths in a cluster\n");
  printf("\t-I which inner control (0 = no control)\n");
  printf("\t-e error limit in terms of \\delta_r\n");
  printf("\t-F error floor (error acceptable whatever \\delta_r)\n");
  printf("\nRates and variance are for T=1.\n");
  printf("All parameters are required.\n\n");

  exit(1);
}
